@@ -64,16 +64,12 @@ record_set_username (UTMP       *u,
                      const char *username)
 {
 #if defined(HAVE_UT_UT_USER)
-        strncpy (u->ut_user,
-                 username,
-                 sizeof (u->ut_user));
+        memccpy (u->ut_user, username, '\0', sizeof (u->ut_user));
         g_debug ("using ut_user %.*s",
                  (int) sizeof (u->ut_user),
                  u->ut_user);
 #elif defined(HAVE_UT_UT_NAME)
-        strncpy (u->ut_name,
-                 username,
-                 sizeof (u->ut_name));
+        memccpy (u->ut_name, username, '\0', sizeof (u->ut_name));
         g_debug ("using ut_name %.*s",
                  (int) sizeof (u->ut_name),
                  u->ut_name);
@@ -84,12 +80,12 @@ static void
 record_set_timestamp (UTMP *u)
 {
 #if defined(HAVE_UT_UT_TV)
-        GTimeVal    now = { 0 };
+        gint64 now;
 
         /* Set time in TV format */
-        g_get_current_time (&now);
-        u->ut_tv.tv_sec  = now.tv_sec;
-        u->ut_tv.tv_usec = now.tv_usec;
+        now = g_get_real_time();
+        u->ut_tv.tv_sec  = now / G_USEC_PER_SEC;
+        u->ut_tv.tv_usec = now % G_USEC_PER_SEC;
         g_debug ("using ut_tv time %ld",
                  (glong) u->ut_tv.tv_sec);
 #elif defined(HAVE_UT_UT_TIME)
@@ -136,7 +132,7 @@ record_set_host (UTMP       *u,
         }
 
         if (hostname != NULL) {
-                strncpy (u->ut_host, hostname, sizeof (u->ut_host));
+                memccpy (u->ut_host, hostname, '\0', sizeof (u->ut_host));
                 g_debug ("using ut_host %.*s", (int) sizeof (u->ut_host), u->ut_host);
 #ifdef HAVE_UT_UT_SYSLEN
                 u->ut_syslen = MIN (strlen (hostname), sizeof (u->ut_host));
@@ -153,17 +149,25 @@ record_set_line (UTMP       *u,
 {
         /*
          * Set ut_line to the device name associated with this display
-         * but remove the "/dev/" prefix.  If no device, then use the
-         * $DISPLAY value.
+         * but remove the "/dev/" prefix if there is one. Otherwise, if it
+         * seems like the display device is a seat id, just use it wholesale.
+         * If there's no device at all, but $DISPLAY is set, just fall back to
+         * using that.
          */
-        if (display_device != NULL
-            && g_str_has_prefix (display_device, "/dev/")) {
-                strncpy (u->ut_line,
+        if (display_device != NULL && g_str_has_prefix (display_device, "/dev/")) {
+                memccpy (u->ut_line,
                          display_device + strlen ("/dev/"),
+                         '\0',
+                         sizeof (u->ut_line));
+        } else if (display_device != NULL && g_str_has_prefix (display_device, "seat")) {
+                memccpy (u->ut_line,
+                         display_device,
+                         '\0',
                          sizeof (u->ut_line));
         } else if (x11_display_name != NULL) {
-                strncpy (u->ut_line,
+                memccpy (u->ut_line,
                          x11_display_name,
+                         '\0',
                          sizeof (u->ut_line));
         }
 
@@ -213,7 +217,9 @@ gdm_session_record_login (GPid                  session_pid,
         /* Handle utmp */
 #if defined(HAVE_GETUTXENT)
         g_debug ("Adding or updating utmp record for login");
+        setutxent();
         pututxline (&session_record);
+        endutxent();
 #elif defined(HAVE_LOGIN)
 	login (&session_record);
 #endif
@@ -256,7 +262,9 @@ gdm_session_record_logout (GPid                  session_pid,
         /* Handle utmp */
 #if defined(HAVE_GETUTXENT)
         g_debug ("Adding or updating utmp record for logout");
+        setutxent();
         pututxline (&session_record);
+        endutxent();
 #elif defined(HAVE_LOGOUT)
         logout (session_record.ut_line);
 #endif
