@@ -34,13 +34,14 @@
 #include "gdm-client-glue.h"
 #include "gdm-manager-glue.h"
 
-#define GDM_CLIENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_CLIENT, GdmClientPrivate))
-
 #define SESSION_DBUS_PATH      "/org/gnome/DisplayManager/Session"
 
-struct GdmClientPrivate
+struct _GdmClient
 {
+        GObject             parent;
+
         GdmUserVerifier    *user_verifier;
+        GdmUserVerifier    *user_verifier_for_reauth;
         GHashTable         *user_verifier_extensions;
 
         GdmGreeter         *greeter;
@@ -50,8 +51,6 @@ struct GdmClientPrivate
         char              **enabled_extensions;
 };
 
-static void     gdm_client_class_init  (GdmClientClass *klass);
-static void     gdm_client_init        (GdmClient      *client);
 static void     gdm_client_finalize    (GObject        *object);
 
 G_DEFINE_TYPE (GdmClient, gdm_client, G_TYPE_OBJECT);
@@ -74,14 +73,14 @@ gdm_client_get_open_connection (GdmClient *client)
 {
         GDBusProxy *proxy = NULL;
 
-        if (client->priv->user_verifier != NULL) {
-                proxy = G_DBUS_PROXY (client->priv->user_verifier);
-        } else if (client->priv->greeter != NULL) {
-                proxy = G_DBUS_PROXY (client->priv->greeter);
-        } else if (client->priv->remote_greeter != NULL) {
-                proxy = G_DBUS_PROXY (client->priv->remote_greeter);
-        } else if (client->priv->chooser != NULL) {
-                proxy = G_DBUS_PROXY (client->priv->chooser);
+        if (client->user_verifier != NULL) {
+                proxy = G_DBUS_PROXY (client->user_verifier);
+        } else if (client->greeter != NULL) {
+                proxy = G_DBUS_PROXY (client->greeter);
+        } else if (client->remote_greeter != NULL) {
+                proxy = G_DBUS_PROXY (client->remote_greeter);
+        } else if (client->chooser != NULL) {
+                proxy = G_DBUS_PROXY (client->chooser);
         }
 
         if (proxy != NULL) {
@@ -97,11 +96,9 @@ on_got_manager (GObject             *object,
                 gpointer             user_data)
 {
         g_autoptr(GTask)      task = user_data;
-        g_autoptr(GdmClient)  client = NULL;
         g_autoptr(GdmManager) manager = NULL;
         g_autoptr(GError)     error = NULL;
 
-        client = GDM_CLIENT (g_async_result_get_source_object (G_ASYNC_RESULT (task)));
         manager = gdm_manager_proxy_new_finish (result, &error);
 
         if (error != NULL) {
@@ -174,11 +171,13 @@ static void
 maybe_complete_user_verifier_proxy_operation (GdmClient          *client,
                                               UserVerifierData   *data)
 {
+        GHashTable *user_verifier_extensions;
         GHashTableIter iter;
         gpointer key, value;
 
-        if (client->priv->user_verifier_extensions != NULL) {
-                g_hash_table_iter_init (&iter, client->priv->user_verifier_extensions);
+        user_verifier_extensions = g_object_get_data (G_OBJECT (data->user_verifier), "gdm-client-user-verifier-extensions");
+        if (user_verifier_extensions != NULL) {
+                g_hash_table_iter_init (&iter, user_verifier_extensions);
                 while (g_hash_table_iter_next (&iter, &key, &value)) {
                         if (value == NULL)
                                 return;
@@ -193,19 +192,21 @@ on_user_verifier_choice_list_proxy_created (GObject            *source,
                                             GAsyncResult       *result,
                                             UserVerifierData   *data)
 {
-        GdmClient                 *client;
+        GHashTable *user_verifier_extensions;
+        g_autoptr(GdmClient)       client = NULL;
         GdmUserVerifierChoiceList *choice_list;
         g_autoptr(GError)          error = NULL;
 
         client = GDM_CLIENT (g_async_result_get_source_object (G_ASYNC_RESULT (data->task)));
 
+        user_verifier_extensions = g_object_get_data (G_OBJECT (data->user_verifier), "gdm-client-user-verifier-extensions");
         choice_list = gdm_user_verifier_choice_list_proxy_new_finish (result, &error);
 
         if (choice_list == NULL) {
                 g_debug ("Couldn't create UserVerifier ChoiceList proxy: %s", error->message);
-                g_hash_table_remove (client->priv->user_verifier_extensions, gdm_user_verifier_choice_list_interface_info ()->name);
+                g_hash_table_remove (user_verifier_extensions, gdm_user_verifier_choice_list_interface_info ()->name);
         } else {
-                g_hash_table_replace (client->priv->user_verifier_extensions, gdm_user_verifier_choice_list_interface_info ()->name, choice_list);
+                g_hash_table_replace (user_verifier_extensions, gdm_user_verifier_choice_list_interface_info ()->name, choice_list);
         }
 
         maybe_complete_user_verifier_proxy_operation (client, data);
@@ -216,7 +217,8 @@ on_user_verifier_extensions_enabled (GdmUserVerifier    *user_verifier,
                                      GAsyncResult       *result,
                                      UserVerifierData   *data)
 {
-        GdmClient *client;
+        g_autoptr(GdmClient)       client = NULL;
+        GHashTable *user_verifier_extensions;
         GCancellable *cancellable;
         GDBusConnection *connection;
         g_autoptr(GError) error = NULL;
@@ -224,6 +226,7 @@ on_user_verifier_extensions_enabled (GdmUserVerifier    *user_verifier,
 
         client = GDM_CLIENT (g_async_result_get_source_object (G_ASYNC_RESULT (data->task)));
         cancellable = g_task_get_cancellable (data->task);
+        user_verifier_extensions = g_object_get_data (G_OBJECT (user_verifier), "gdm-client-user-verifier-extensions");
 
         gdm_user_verifier_call_enable_extensions_finish (user_verifier, result, &error);
 
@@ -236,13 +239,13 @@ on_user_verifier_extensions_enabled (GdmUserVerifier    *user_verifier,
 
         connection = g_dbus_proxy_get_connection (G_DBUS_PROXY (user_verifier));
 
-        for (i = 0; client->priv->enabled_extensions[i] != NULL; i++) {
-                g_debug ("Enabled extensions[%lu] = %s", i, client->priv->enabled_extensions[i]);
-                g_hash_table_insert (client->priv->user_verifier_extensions, client->priv->enabled_extensions[i], NULL);
+        for (i = 0; client->enabled_extensions[i] != NULL; i++) {
+                g_debug ("Enabled extensions[%lu] = %s", i, client->enabled_extensions[i]);
+                g_hash_table_insert (user_verifier_extensions, client->enabled_extensions[i], NULL);
 
-                if (strcmp (client->priv->enabled_extensions[i],
+                if (strcmp (client->enabled_extensions[i],
                             gdm_user_verifier_choice_list_interface_info ()->name) == 0) {
-                        g_hash_table_insert (client->priv->user_verifier_extensions, client->priv->enabled_extensions[i], NULL);
+                        g_hash_table_insert (user_verifier_extensions, client->enabled_extensions[i], NULL);
                         gdm_user_verifier_choice_list_proxy_new (connection,
                                                                  G_DBUS_PROXY_FLAGS_NONE,
                                                                  NULL,
@@ -252,13 +255,13 @@ on_user_verifier_extensions_enabled (GdmUserVerifier    *user_verifier,
                                                                  on_user_verifier_choice_list_proxy_created,
                                                                  data);
                 } else {
-                        g_debug ("User verifier extension %s is unsupported", client->priv->enabled_extensions[i]);
-                        g_hash_table_remove (client->priv->user_verifier_extensions,
-                                             client->priv->enabled_extensions[i]);
+                        g_debug ("User verifier extension %s is unsupported", client->enabled_extensions[i]);
+                        g_hash_table_remove (user_verifier_extensions,
+                                             client->enabled_extensions[i]);
                 }
         }
 
-        if (g_hash_table_size (client->priv->user_verifier_extensions) == 0) {
+        if (g_hash_table_size (user_verifier_extensions) == 0) {
                 g_debug ("No supported user verifier extensions");
                 complete_user_verifier_proxy_operation (client, data);
         }
@@ -279,7 +282,8 @@ on_user_verifier_proxy_created (GObject            *source,
                                 GAsyncResult       *result,
                                 gpointer            user_data)
 {
-        GdmClient       *self;
+        g_autoptr(GdmClient)       self = NULL;
+        GHashTable         *user_verifier_extensions;
         GCancellable    *cancellable = NULL;
         g_autoptr(GdmUserVerifier) user_verifier = NULL;
         g_autoptr(GTask)           task = user_data;
@@ -294,7 +298,7 @@ on_user_verifier_proxy_created (GObject            *source,
         g_debug ("UserVerifier %p created", user_verifier);
 
         self = GDM_CLIENT (g_async_result_get_source_object (G_ASYNC_RESULT (task)));
-        if (self->priv->enabled_extensions == NULL) {
+        if (self->enabled_extensions == NULL) {
                 g_debug ("no enabled extensions");
                 g_task_return_pointer (task,
                                        g_steal_pointer (&user_verifier),
@@ -302,15 +306,19 @@ on_user_verifier_proxy_created (GObject            *source,
                 return;
         }
 
-        self->priv->user_verifier_extensions = g_hash_table_new_full (g_str_hash,
-                                                                      g_str_equal,
-                                                                      NULL,
-                                                                      (GDestroyNotify)
-                                                                      free_interface_skeleton);
+        user_verifier_extensions = g_hash_table_new_full (g_str_hash,
+                                                          g_str_equal,
+                                                          NULL,
+                                                          (GDestroyNotify)
+                                                          free_interface_skeleton);
+        g_object_set_data_full (G_OBJECT (user_verifier),
+                                "gdm-client-user-verifier-extensions",
+                                user_verifier_extensions,
+                                (GDestroyNotify) g_hash_table_unref);
         cancellable = g_task_get_cancellable (task);
         gdm_user_verifier_call_enable_extensions (user_verifier,
                                                   (const char * const *)
-                                                  self->priv->enabled_extensions,
+                                                  self->enabled_extensions,
                                                   cancellable,
                                                   (GAsyncReadyCallback)
                                                   on_user_verifier_extensions_enabled,
@@ -446,80 +454,6 @@ gdm_client_get_connection_sync (GdmClient      *client,
         return connection;
 }
 
-static void
-on_connected (GObject            *source_object,
-              GAsyncResult       *result,
-              gpointer            user_data)
-{
-        g_autoptr(GTask)           task = user_data;
-        g_autoptr(GDBusConnection) connection = NULL;
-        g_autoptr(GError)          error = NULL;
-
-        connection = g_dbus_connection_new_for_address_finish (result, &error);
-        if (!connection) {
-                g_task_return_error (task, g_steal_pointer (&error));
-                return;
-        }
-
-        g_task_return_pointer (task,
-                               g_steal_pointer (&connection),
-                               (GDestroyNotify) g_object_unref);
-}
-
-static void
-on_session_opened (GdmManager         *manager,
-                   GAsyncResult       *result,
-                   gpointer            user_data)
-{
-        GCancellable     *cancellable;
-        g_autoptr(GTask)     task = user_data;
-        g_autoptr(GdmClient) client = NULL;
-        g_autoptr(GError)    error = NULL;
-        g_autofree char     *address = NULL;
-
-        client = GDM_CLIENT (g_async_result_get_source_object (G_ASYNC_RESULT (task)));
-
-        if (!gdm_manager_call_open_session_finish (manager,
-                                                   &address,
-                                                   result,
-                                                   &error)) {
-                g_task_return_error (task, g_steal_pointer (&error));
-                return;
-        }
-
-        cancellable = g_task_get_cancellable (task);
-        g_dbus_connection_new_for_address (address,
-                                           G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_CLIENT,
-                                           NULL,
-                                           cancellable,
-                                           on_connected,
-                                           g_steal_pointer (&task));
-}
-
-static void
-on_got_manager_for_opening_connection (GdmClient           *client,
-                                       GAsyncResult        *result,
-                                       gpointer             user_data)
-{
-        GCancellable *cancellable;
-        g_autoptr(GTask)      task = user_data;
-        g_autoptr(GdmManager) manager = NULL;
-        g_autoptr(GError)     error = NULL;
-
-        manager = g_task_propagate_pointer (G_TASK (result), &error);
-        if (manager == NULL) {
-                g_task_return_error (task, g_steal_pointer (&error));
-                return;
-        }
-
-        cancellable = g_task_get_cancellable (task);
-        gdm_manager_call_open_session (manager,
-                                       cancellable,
-                                       (GAsyncReadyCallback)
-                                       on_session_opened,
-                                       g_steal_pointer (&task));
-}
-
 static GDBusConnection *
 gdm_client_get_connection_finish (GdmClient      *client,
                                   GAsyncResult   *result,
@@ -544,6 +478,7 @@ gdm_client_get_connection (GdmClient           *client,
                             gpointer             user_data)
 {
         g_autoptr(GTask) task = NULL;
+        g_autoptr(GError) error = NULL;
         GDBusConnection *connection;
 
         g_return_if_fail (GDM_IS_CLIENT (client));
@@ -553,19 +488,16 @@ gdm_client_get_connection (GdmClient           *client,
                            callback,
                            user_data);
 
-        connection = gdm_client_get_open_connection (client);
+        connection = gdm_client_get_connection_sync (client,
+                                                     cancellable,
+                                                     &error);
         if (connection != NULL) {
-            g_task_return_pointer (task,
-                                   g_object_ref (connection),
-                                   (GDestroyNotify) g_object_unref);
-            return;
+                g_task_return_pointer (task,
+                                       g_object_ref (connection),
+                                       (GDestroyNotify) g_object_unref);
+        } else {
+                g_task_return_error (task, g_steal_pointer (&error));
         }
-
-        get_manager (client,
-                     cancellable,
-                     (GAsyncReadyCallback)
-                     on_got_manager_for_opening_connection,
-                     g_steal_pointer (&task));
 }
 
 /**
@@ -636,6 +568,8 @@ gdm_client_open_reauthentication_channel_sync (GdmClient     *client,
                                                           cancellable,
                                                           error);
 
+        g_set_weak_pointer (&client->user_verifier_for_reauth, user_verifier);
+
         return user_verifier;
 }
 
@@ -695,9 +629,15 @@ gdm_client_open_reauthentication_channel_finish (GdmClient       *client,
                                                  GAsyncResult    *result,
                                                  GError         **error)
 {
+        GdmUserVerifier *user_verifier;
+
         g_return_val_if_fail (GDM_IS_CLIENT (client), NULL);
 
-        return g_task_propagate_pointer (G_TASK (result), error);
+        user_verifier = g_task_propagate_pointer (G_TASK (result), error);
+
+        g_set_weak_pointer (&client->user_verifier_for_reauth, user_verifier);
+
+        return user_verifier;
 }
 
 /**
@@ -717,9 +657,10 @@ gdm_client_get_user_verifier_sync (GdmClient     *client,
                                    GError       **error)
 {
         g_autoptr(GDBusConnection) connection = NULL;
+        GdmUserVerifier *user_verifier;
 
-        if (client->priv->user_verifier != NULL) {
-                return g_object_ref (client->priv->user_verifier);
+        if (client->user_verifier != NULL) {
+                return g_object_ref (client->user_verifier);
         }
 
         connection = gdm_client_get_connection_sync (client, cancellable, error);
@@ -728,35 +669,40 @@ gdm_client_get_user_verifier_sync (GdmClient     *client,
                 return NULL;
         }
 
-        client->priv->user_verifier = gdm_user_verifier_proxy_new_sync (connection,
-                                                                        G_DBUS_PROXY_FLAGS_NONE,
-                                                                        NULL,
-                                                                        SESSION_DBUS_PATH,
-                                                                        cancellable,
-                                                                        error);
+        user_verifier = gdm_user_verifier_proxy_new_sync (connection,
+                                                          G_DBUS_PROXY_FLAGS_NONE,
+                                                          NULL,
+                                                          SESSION_DBUS_PATH,
+                                                          cancellable,
+                                                          error);
 
-        if (client->priv->user_verifier != NULL) {
-                g_object_add_weak_pointer (G_OBJECT (client->priv->user_verifier),
-                                           (gpointer *)
-                                           &client->priv->user_verifier);
-                if (client->priv->enabled_extensions != NULL) {
+        g_set_weak_pointer (&client->user_verifier, user_verifier);
+
+        if (client->user_verifier != NULL) {
+                if (client->enabled_extensions != NULL) {
+                        GHashTable *user_verifier_extensions;
                         gboolean res;
 
-                        client->priv->user_verifier_extensions = g_hash_table_new_full (g_str_hash,
-                                                                                        g_str_equal,
-                                                                                        NULL,
-                                                                                        (GDestroyNotify)
-                                                                                        free_interface_skeleton);
-                        res = gdm_user_verifier_call_enable_extensions_sync (client->priv->user_verifier,
+                        user_verifier_extensions = g_hash_table_new_full (g_str_hash,
+                                                                          g_str_equal,
+                                                                          NULL,
+                                                                          (GDestroyNotify)
+                                                                          free_interface_skeleton);
+                        g_object_set_data_full (G_OBJECT (client->user_verifier),
+                                                "gdm-client-user-verifier-extensions",
+                                                user_verifier_extensions,
+                                                (GDestroyNotify) g_hash_table_unref);
+
+                        res = gdm_user_verifier_call_enable_extensions_sync (client->user_verifier,
                                                                             (const char * const *)
-                                                                             client->priv->enabled_extensions,
+                                                                             client->enabled_extensions,
                                                                              cancellable,
                                                                              NULL);
 
                         if (res) {
                                 size_t i;
-                                for (i = 0; client->priv->enabled_extensions[i] != NULL; i++) {
-                                            if (strcmp (client->priv->enabled_extensions[i],
+                                for (i = 0; client->enabled_extensions[i] != NULL; i++) {
+                                            if (strcmp (client->enabled_extensions[i],
                                                         gdm_user_verifier_choice_list_interface_info ()->name) == 0) {
                                                         GdmUserVerifierChoiceList *choice_list_interface;
                                                         choice_list_interface = gdm_user_verifier_choice_list_proxy_new_sync (connection,
@@ -766,14 +712,14 @@ gdm_client_get_user_verifier_sync (GdmClient     *client,
                                                                                                                               cancellable,
                                                                                                                               NULL);
                                                         if (choice_list_interface != NULL)
-                                                                    g_hash_table_insert (client->priv->user_verifier_extensions, client->priv->enabled_extensions[i], choice_list_interface);
+                                                                    g_hash_table_insert (user_verifier_extensions, client->enabled_extensions[i], choice_list_interface);
                                             }
                                 }
                         }
                 }
         }
 
-        return client->priv->user_verifier;
+        return client->user_verifier;
 }
 
 static void
@@ -827,9 +773,9 @@ gdm_client_get_user_verifier (GdmClient           *client,
                            callback,
                            user_data);
 
-        if (client->priv->user_verifier != NULL) {
+        if (client->user_verifier != NULL) {
                 g_task_return_pointer (task,
-                                       g_object_ref (client->priv->user_verifier),
+                                       g_object_ref (client->user_verifier),
                                        (GDestroyNotify) g_object_unref);
                 return;
         }
@@ -861,18 +807,14 @@ gdm_client_get_user_verifier_finish (GdmClient       *client,
 
         g_return_val_if_fail (GDM_IS_CLIENT (client), NULL);
 
-        if (client->priv->user_verifier != NULL)
-                return g_object_ref (client->priv->user_verifier);
+        if (client->user_verifier != NULL)
+                return g_object_ref (client->user_verifier);
 
         user_verifier = g_task_propagate_pointer (G_TASK (result), error);
         if (user_verifier == NULL)
                 return NULL;
 
-        client->priv->user_verifier = user_verifier;
-
-        g_object_add_weak_pointer (G_OBJECT (client->priv->user_verifier),
-                                   (gpointer *)
-                                   &client->priv->user_verifier);
+        g_set_weak_pointer (&client->user_verifier, user_verifier);
 
         return user_verifier;
 }
@@ -890,10 +832,18 @@ gdm_client_get_user_verifier_finish (GdmClient       *client,
 GdmUserVerifierChoiceList *
 gdm_client_get_user_verifier_choice_list (GdmClient *client)
 {
-        if (client->priv->user_verifier_extensions == NULL)
+        GHashTable *user_verifier_extensions = NULL;
+
+        if (client->user_verifier_for_reauth != NULL)
+                user_verifier_extensions = g_object_get_data (G_OBJECT (client->user_verifier_for_reauth), "gdm-client-user-verifier-extensions");
+
+        if (user_verifier_extensions == NULL && client->user_verifier != NULL)
+                user_verifier_extensions = g_object_get_data (G_OBJECT (client->user_verifier), "gdm-client-user-verifier-extensions");
+
+        if (user_verifier_extensions == NULL)
                 return NULL;
 
-        return g_hash_table_lookup (client->priv->user_verifier_extensions,
+        return g_hash_table_lookup (user_verifier_extensions,
                                     gdm_user_verifier_choice_list_interface_info ()->name);
 }
 
@@ -991,9 +941,9 @@ gdm_client_get_greeter (GdmClient           *client,
                            callback,
                            user_data);
 
-        if (client->priv->greeter != NULL) {
+        if (client->greeter != NULL) {
                 g_task_return_pointer (task,
-                                       g_object_ref (client->priv->greeter),
+                                       g_object_ref (client->greeter),
                                        (GDestroyNotify) g_object_unref);
                 return;
         }
@@ -1025,18 +975,14 @@ gdm_client_get_greeter_finish (GdmClient       *client,
 
         g_return_val_if_fail (GDM_IS_CLIENT (client), NULL);
 
-        if (client->priv->greeter != NULL)
-                return g_object_ref (client->priv->greeter);
+        if (client->greeter != NULL)
+                return g_object_ref (client->greeter);
 
         greeter = g_task_propagate_pointer (G_TASK (result), error);
         if (greeter == NULL)
                 return NULL;
 
-        client->priv->greeter = greeter;
-
-        g_object_add_weak_pointer (G_OBJECT (client->priv->greeter),
-                                   (gpointer *)
-                                   &client->priv->greeter);
+        g_set_weak_pointer (&client->greeter, greeter);
 
         return greeter;
 }
@@ -1060,9 +1006,10 @@ gdm_client_get_greeter_sync (GdmClient     *client,
                              GError       **error)
 {
         g_autoptr(GDBusConnection) connection = NULL;
+        GdmGreeter *greeter;
 
-        if (client->priv->greeter != NULL) {
-                return g_object_ref (client->priv->greeter);
+        if (client->greeter != NULL) {
+                return g_object_ref (client->greeter);
         }
 
         connection = gdm_client_get_connection_sync (client, cancellable, error);
@@ -1071,22 +1018,20 @@ gdm_client_get_greeter_sync (GdmClient     *client,
                 return NULL;
         }
 
-        client->priv->greeter = gdm_greeter_proxy_new_sync (connection,
-                                                            G_DBUS_PROXY_FLAGS_NONE,
-                                                            NULL,
-                                                            SESSION_DBUS_PATH,
-                                                            cancellable,
-                                                            error);
+        greeter = gdm_greeter_proxy_new_sync (connection,
+                                              G_DBUS_PROXY_FLAGS_NONE,
+                                              NULL,
+                                              SESSION_DBUS_PATH,
+                                              cancellable,
+                                              error);
 
-        if (client->priv->greeter != NULL) {
-                g_object_add_weak_pointer (G_OBJECT (client->priv->greeter),
-                                           (gpointer *)
-                                           &client->priv->greeter);
+        g_set_weak_pointer (&client->greeter, greeter);
 
-                query_for_timed_login_requested_signal (client->priv->greeter);
+        if (client->greeter != NULL) {
+                query_for_timed_login_requested_signal (client->greeter);
         }
 
-        return client->priv->greeter;
+        return client->greeter;
 }
 
 static void
@@ -1161,9 +1106,9 @@ gdm_client_get_remote_greeter (GdmClient           *client,
                            callback,
                            user_data);
 
-        if (client->priv->remote_greeter != NULL) {
+        if (client->remote_greeter != NULL) {
                 g_task_return_pointer (task,
-                                       g_object_ref (client->priv->remote_greeter),
+                                       g_object_ref (client->remote_greeter),
                                        (GDestroyNotify) g_object_unref);
                 return;
         }
@@ -1195,18 +1140,14 @@ gdm_client_get_remote_greeter_finish (GdmClient     *client,
 
         g_return_val_if_fail (GDM_IS_CLIENT (client), NULL);
 
-        if (client->priv->remote_greeter != NULL)
-                return g_object_ref (client->priv->remote_greeter);
+        if (client->remote_greeter != NULL)
+                return g_object_ref (client->remote_greeter);
 
         remote_greeter = g_task_propagate_pointer (G_TASK (result), error);
         if (remote_greeter == NULL)
                 return NULL;
 
-        client->priv->remote_greeter = remote_greeter;
-
-        g_object_add_weak_pointer (G_OBJECT (client->priv->remote_greeter),
-                                   (gpointer *)
-                                   &client->priv->remote_greeter);
+        g_set_weak_pointer (&client->remote_greeter, remote_greeter);
 
         return remote_greeter;
 }
@@ -1229,9 +1170,10 @@ gdm_client_get_remote_greeter_sync (GdmClient     *client,
                                     GError       **error)
 {
         g_autoptr(GDBusConnection) connection = NULL;
+        GdmRemoteGreeter *remote_greeter;
 
-        if (client->priv->remote_greeter != NULL) {
-                return g_object_ref (client->priv->remote_greeter);
+        if (client->remote_greeter != NULL) {
+                return g_object_ref (client->remote_greeter);
         }
 
         connection = gdm_client_get_connection_sync (client, cancellable, error);
@@ -1240,20 +1182,16 @@ gdm_client_get_remote_greeter_sync (GdmClient     *client,
                 return NULL;
         }
 
-        client->priv->remote_greeter = gdm_remote_greeter_proxy_new_sync (connection,
-                                                                          G_DBUS_PROXY_FLAGS_NONE,
-                                                                          NULL,
-                                                                          SESSION_DBUS_PATH,
-                                                                          cancellable,
-                                                                          error);
+        remote_greeter = gdm_remote_greeter_proxy_new_sync (connection,
+                                                            G_DBUS_PROXY_FLAGS_NONE,
+                                                            NULL,
+                                                            SESSION_DBUS_PATH,
+                                                            cancellable,
+                                                            error);
 
-        if (client->priv->remote_greeter != NULL) {
-                g_object_add_weak_pointer (G_OBJECT (client->priv->remote_greeter),
-                                           (gpointer *)
-                                           &client->priv->remote_greeter);
-        }
+        g_set_weak_pointer (&client->remote_greeter, remote_greeter);
 
-        return client->priv->remote_greeter;
+        return client->remote_greeter;
 }
 
 static void
@@ -1329,9 +1267,9 @@ gdm_client_get_chooser (GdmClient           *client,
                            callback,
                            user_data);
 
-        if (client->priv->chooser != NULL) {
+        if (client->chooser != NULL) {
                 g_task_return_pointer (task,
-                                       g_object_ref (client->priv->chooser),
+                                       g_object_ref (client->chooser),
                                        (GDestroyNotify) g_object_unref);
                 return;
         }
@@ -1363,18 +1301,14 @@ gdm_client_get_chooser_finish (GdmClient       *client,
 
         g_return_val_if_fail (GDM_IS_CLIENT (client), NULL);
 
-        if (client->priv->chooser != NULL)
-                return g_object_ref (client->priv->chooser);
+        if (client->chooser != NULL)
+                return g_object_ref (client->chooser);
 
         chooser = g_task_propagate_pointer (G_TASK (result), error);
         if (chooser == NULL)
                 return NULL;
 
-        client->priv->chooser = chooser;
-
-        g_object_add_weak_pointer (G_OBJECT (client->priv->chooser),
-                                   (gpointer *)
-                                   &client->priv->chooser);
+        g_set_weak_pointer (&client->chooser, chooser);
 
         return chooser;
 }
@@ -1397,9 +1331,10 @@ gdm_client_get_chooser_sync (GdmClient     *client,
                              GError       **error)
 {
         g_autoptr(GDBusConnection) connection = NULL;
+        GdmChooser *chooser;
 
-        if (client->priv->chooser != NULL) {
-                return g_object_ref (client->priv->chooser);
+        if (client->chooser != NULL) {
+                return g_object_ref (client->chooser);
         }
 
         connection = gdm_client_get_connection_sync (client, cancellable, error);
@@ -1408,20 +1343,16 @@ gdm_client_get_chooser_sync (GdmClient     *client,
                 return NULL;
         }
 
-        client->priv->chooser = gdm_chooser_proxy_new_sync (connection,
-                                                            G_DBUS_PROXY_FLAGS_NONE,
-                                                            NULL,
-                                                            SESSION_DBUS_PATH,
-                                                            cancellable,
-                                                            error);
+        chooser = gdm_chooser_proxy_new_sync (connection,
+                                              G_DBUS_PROXY_FLAGS_NONE,
+                                              NULL,
+                                              SESSION_DBUS_PATH,
+                                              cancellable,
+                                              error);
 
-        if (client->priv->chooser != NULL) {
-                g_object_add_weak_pointer (G_OBJECT (client->priv->chooser),
-                                           (gpointer *)
-                                           &client->priv->chooser);
-        }
+        g_set_weak_pointer (&client->chooser, chooser);
 
-        return client->priv->chooser;
+        return client->chooser;
 }
 
 static void
@@ -1430,16 +1361,11 @@ gdm_client_class_init (GdmClientClass *klass)
         GObjectClass   *object_class = G_OBJECT_CLASS (klass);
 
         object_class->finalize = gdm_client_finalize;
-
-        g_type_class_add_private (klass, sizeof (GdmClientPrivate));
-
 }
 
 static void
 gdm_client_init (GdmClient *client)
 {
-
-        client->priv = GDM_CLIENT_GET_PRIVATE (client);
 }
 
 static void
@@ -1452,33 +1378,15 @@ gdm_client_finalize (GObject *object)
 
         client = GDM_CLIENT (object);
 
-        g_return_if_fail (client->priv != NULL);
+        g_return_if_fail (client != NULL);
 
-        if (client->priv->user_verifier != NULL) {
-                g_object_remove_weak_pointer (G_OBJECT (client->priv->user_verifier),
-                                              (gpointer *)
-                                              &client->priv->user_verifier);
-        }
+        g_clear_weak_pointer (&client->user_verifier);
+        g_clear_weak_pointer (&client->user_verifier_for_reauth);
+        g_clear_weak_pointer (&client->greeter);
+        g_clear_weak_pointer (&client->remote_greeter);
+        g_clear_weak_pointer (&client->chooser);
 
-        if (client->priv->greeter != NULL) {
-                g_object_remove_weak_pointer (G_OBJECT (client->priv->greeter),
-                                              (gpointer *)
-                                              &client->priv->greeter);
-        }
-
-        if (client->priv->remote_greeter != NULL) {
-                g_object_remove_weak_pointer (G_OBJECT (client->priv->remote_greeter),
-                                              (gpointer *)
-                                              &client->priv->remote_greeter);
-        }
-
-        if (client->priv->chooser != NULL) {
-                g_object_remove_weak_pointer (G_OBJECT (client->priv->chooser),
-                                              (gpointer *)
-                                              &client->priv->chooser);
-        }
-
-        g_strfreev (client->priv->enabled_extensions);
+        g_strfreev (client->enabled_extensions);
 
         G_OBJECT_CLASS (gdm_client_parent_class)->finalize (object);
 }
@@ -1510,5 +1418,5 @@ void
 gdm_client_set_enabled_extensions (GdmClient          *client,
                                    const char * const *extensions)
 {
-        client->priv->enabled_extensions = g_strdupv ((char **) extensions);
+        client->enabled_extensions = g_strdupv ((char **) extensions);
 }
